// PhotoCGI.cpp - Copyright 2003 Robert Basler.  This is only partial source code.  This program uses the CGI
// libraries from the book: 
//
// "HTML 3.2 Plus The Definitive HTML 3.2 Problem-Solver" David Kerven, Jeff Fouse, John Zakour, Waite Group Press,
// 1997, ISBN 1-57169-091-3


#include "common.h"
#include "system.h"
#include "inputs.h"
#include <stdio.h>
#include "wstring.h"
#include "timing.h"
#include "memman.h"

// See photocgi.txt for detailed usage instructions.

#define Version "1.01 (2003 September 27)"
#define Copyright "Copyright 2003 Robert Basler, All Rights Reserved."

#define MAX_DOWNLOADS 9     // Maximum number of downloadable versions of each photo
#define IP_LEN 20           // length of ip address string
#define PARM_LEN 255        // length of individual parameters
#define SCR_LEN 512         // length of command line


// Downloadable picture structure
typedef struct SDownload {
     char downloadname[SYS_MAXFILENAMELENGTH+1]; // Downloadable version file name
     char description[256];                      // Description of file
} TDownload;


// Info structure for each photo in the setting file
typedef struct SPhoto {
     char thumbname[SYS_MAXFILENAMELENGTH+1];    // Thumbnail photo file name
     char previewname[SYS_MAXFILENAMELENGTH+1];  // Preview photo file name
     char description[256];                      // Description of photo
     int32 downloadcount;                          // Number of download records in use
     TDownload downloads[MAX_DOWNLOADS];         // Downloadable file records
} TPhoto;


TPhoto *Photos = NULL;   // Global pointer to all photo records

char settingsfn[SYS_MAXFILENAMELENGTH+1], logfn[SYS_MAXFILENAMELENGTH+1], masterfn[SYS_MAXFILENAMELENGTH+1], errfn[SYS_MAXFILENAMELENGTH+1];
char ip[IP_LEN+1], scriptname[SCR_LEN+1];
int32 ibase, ilength, inputlength, iselected, iphotocount;


void spudspaces(char *buf) {
// strip any leading spaces
char *a=buf, *b=buf;
while (((*a)!='\0') && ((*b)==' ')) {
     *a=*b;
     b++;
};
while ((*a)!='\0') {
     *a=*b;
     a++;
     b++;
};
};


void spudallspaces(char *buf) {
// strip all space characters from a line
char *a=buf, *b=buf;
while ((*a)!='\0') {
     *a=*b;
     if (*a!=' ') a++;
     b++;
};

};

void spudchars(char *buf) {
// strip any leading nonspaces
char *a=buf, *b=buf;
while (((*a)!='\0') && ((*b)!=' ')) {
     *a=*b;
     b++;
};
while ((*a)!='\0') {
     *a=*b;
     a++;
     b++;
};
*a='\0';
};


int getint(char *buf,int *intval) {
// get rid of any leading spaces
spudspaces(buf);
if (*buf=='\0') return(-1);
if (sscanf((char*)buf,"%i",intval)!=1) return(-1);
// now delete the number
spudchars(buf);
return(0);
};


int getdword(char *buf,dword *intval) {
// get rid of any leading spaces
spudspaces(buf);
if (*buf=='\0') return(-1);
if (sscanf((char*)buf,"%li",intval)!=1) return(-1);
// now delete the number
spudchars(buf);
return(0);
};


int getstring(char *buf, char *st) {
char lc;
spudspaces(buf);
if (*buf=='\0') return(-1);
if (*buf=='"') {
     char *b=buf+1, *a=st;
     // copy until double quote
     lc='\0';
     while (((*b)!='\0') && (((*b)!='"') || ((*b=='"') && (lc=='\\')))) {
          *a=*b;
          // Allow \" to be entered for a double quote.
          // and \\ for \.
          if (lc=='\\') {
               switch(*b) {
                    case '"': {
                         lc=*b;
                         a++;
                         break;
                    };
                    case '\\': {
                         lc='\0';
                         a++;
                         break;
                    };
                    case 'n': {
                         lc=*b;
                         *a='\\';
                         a++;
                         *a=*b;
                         a++;
                         break;
                    };
                    default: {
                         lc=*b;
                         a++;
                         break;
                    };
               };
          }
          else {
               if (*b!='\\') a++;
               else lc=*b;
          };
          b++;
     };
     *a='\0';
     if (*b=='"') b++;
     strcpya((char*)buf,(char*)b);
}
else {
     char *b=buf, *a=st;
     // copy until space
     while (((*b)!='\0') && ((*b)!=' ')) {
          *a=*b;
          a++;
          b++;
     };
     *a='\0';
     if (*b==' ') b++;
     strcpya((char*)buf,(char*)b);
}
return(0);
};


// Log the result line
void LogStatus(char *line) {
if (strlena(logfn)>0) {
     FILE *l;
     timevalue t=TimeGet();
     do {
          l=fopen(logfn,"at");
          if (l==NULL) SysSleep(50);
     } while ((l==NULL) && (TimeElapsed(t)<5000)); /* enddo */
     if (l!=NULL) {
          fprintf(l,"%s\n",line);
          fclose(l);
     } /* endif */
};
}


// This routine reads in the file fn and replaces the {result} tag with the provided message.  The resulting
// file is sent to stdout.
void ReturnPage(char *fn, char *msg) {
char *c, *d, *e, buf[1024];
timevalue t;
FILE *f;

#if 0
// Log the result line
if (strlena(logfn)>0) {
     FILE *l;
     t=TimeGet();
     do {
          l=fopen(logfn,"at");
          if (l==NULL) SysSleep(50);
     } while ((l==NULL) && (TimeElapsed(t)<5000)); /* enddo */
     if (l!=NULL) {
          fprintf(l,"%s %s %s\n",ip,TimeStringA(buf,1024),msg);
          fclose(l);
     } /* endif */
};
#endif

fprintf(stdout,"Content-type: text/html\n\n");
t=TimeGet();
do {
     f=fopen(fn,"rt");
     if (f==NULL) SysSleep(50);
} while ((f==NULL) && (TimeElapsed(t)<5000)); /* enddo */
if (f==NULL) {
     fprintf(stdout,"<head></head><body>Unable to open source file %s.<p>Raw message is: %s<P>Please report this to technical support.</body>",fn,msg);
} else {
     while (!feof(f)) {
          if (fgets(buf,1024,f)!=NULL) {
               c=buf;
               while (strlena(c)>0) {
                    e=c;
                    while ((*c!='\0') && (*c!='{')) c++;
                    if (*c=='{') {
                         d=strchra(c,'}');
                         if (d!=NULL) {
                            *d='\0';
                            c++;
                            if (stricmpa(c,"control")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 fprintf(stdout,"%s",settingsfn);
                                 c+=9;
                            } else if (stricmpa(c,"version")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 fprintf(stdout,"%s",Version);
                                 c+=9;
                            } else if (stricmpa(c,"copyright")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 fprintf(stdout,"%s",Copyright);
                                 c+=11;
                            } else if (stricmpa(c,"result")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 fprintf(stdout,"%i",msg);
                                 c+=8;
                            } else if (stricmpa(c,"base")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 fprintf(stdout,"%i",ibase);
                                 c+=6;
                            } else if (stricmpa(c,"length")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 fprintf(stdout,"%i",ilength);
                                 c+=8;
                            } else if (stricmpa(c,"first")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 fprintf(stdout,"%i",ibase+1);
                                 c+=7;
                            } else if (stricmpa(c,"last")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 fprintf(stdout,"%i",ibase+ilength);
                                 c+=6;
                            } else if (stricmpa(c,"selected")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 fprintf(stdout,"%i",iselected);
                                 c+=10;
                            } else if (stricmpa(c,"next")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 if (ibase+ilength>=iphotocount) fprintf(stdout,"%i",ibase);
                                 else fprintf(stdout,"%i",min(ibase+ilength,iphotocount-1));
                                 c+=6;
                            } else if (stricmpa(c,"prev")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 fprintf(stdout,"%i",max(ibase-inputlength,0));
                                 c+=6;
                            } else if (stricmpa(c,"count")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 fprintf(stdout,"%i",iphotocount);
                                 c+=7;
                            } else if (stricmpa(c,"selectedphoto")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 if ((Photos!=NULL) && (ibase+iselected<iphotocount)) {
                                      if (strlena(Photos[ibase+iselected].previewname)>0) fprintf(stdout,"%s",Photos[ibase+iselected].previewname);
                                 }
                                 c+=15;
                            } else if (stricmpa(c,"selecteddescription")==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 if ((Photos!=NULL) && (ibase+iselected<iphotocount)) {
                                      if (strlena(Photos[ibase+iselected].description)>0) fprintf(stdout,"%s",Photos[ibase+iselected].description);
                                 }
                                 c+=21;
                            } else if (strnicmpa(c,"thumbimg",8)==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 c+=10;
                                 int32 thumbid;
                                 if (sscanf(c,"%i",&thumbid)!=1) thumbid=0;
                                 if ((Photos!=NULL) && (ibase+thumbid<iphotocount)) {
                                      if (strlena(Photos[ibase+thumbid].thumbname)>0) fprintf(stdout,"<img src=\"%s\">",Photos[ibase+thumbid].thumbname);
                                 }
                                 c=d;
                                 c++;
                            } else if (strnicmpa(c,"thumb",5)==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 c+=7;
                                 int32 thumbid;
                                 if (sscanf(c,"%i",&thumbid)!=1) thumbid=0;
                                 if ((Photos!=NULL) && (ibase+thumbid<iphotocount)) {
                                      if (strlena(Photos[ibase+thumbid].thumbname)>0) fprintf(stdout,"%s",Photos[ibase+thumbid].thumbname);
                                 }
                                 c=d;
                                 c++;
                            } else if (strnicmpa(c,"description",11)==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 c+=13;
                                 int32 descriptionid;
                                 if (sscanf(c,"%i",&descriptionid)!=1) descriptionid=0;
                                 if ((Photos!=NULL) && (ibase+descriptionid<iphotocount)) {
                                      if (strlena(Photos[ibase+descriptionid].description)>0) fprintf(stdout,"%s",Photos[ibase+descriptionid].description);
                                 }
                                 c=d;
                                 c++;
                            } else if (strnicmpa(c,"downloadlink",12)==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 c+=14;
                                 int32 pictureid, versionid;
                                 if (sscanf(c,"%i %i",&pictureid,&versionid)!=2) {
                                      pictureid=0;
                                      versionid=0;
                                 }
                                 if ((Photos!=NULL) && (ibase+pictureid<iphotocount)) {
                                      if ((strlena(Photos[ibase+pictureid].downloads[versionid].description)>0) && (strlena(Photos[ibase+pictureid].downloads[versionid].downloadname)>0)) fprintf(stdout,"<a href=\"%s\">%s</a>",Photos[ibase+pictureid].downloads[versionid].downloadname,Photos[ibase+pictureid].downloads[versionid].description);
                                 }
                                 c=d;
                                 c++;
                            } else if (strnicmpa(c,"downloadphoto",13)==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 c+=15;
                                 int32 pictureid, versionid;
                                 if (sscanf(c,"%i %i",&pictureid,&versionid)!=2) {
                                      pictureid=0;
                                      versionid=0;
                                 }
                                 if ((Photos!=NULL) && (ibase+pictureid<iphotocount)) {
                                      if (strlena(Photos[ibase+pictureid].downloads[versionid].downloadname)>0) fprintf(stdout,"%s",Photos[ibase+pictureid].downloads[versionid].downloadname);
                                 }
                                 c=d;
                                 c++;
                            } else if (strnicmpa(c,"downloaddesc",12)==0) {
                                 c--;
                                 *c='\0';
                                 fprintf(stdout,"%s",e);
                                 c+=14;
                                 int32 pictureid, versionid;
                                 if (sscanf(c,"%i %i",&pictureid,&versionid)!=2) {
                                      pictureid=0;
                                      versionid=0;
                                 }
                                 if ((Photos!=NULL) && (ibase+pictureid<iphotocount)) {
                                      if (strlena(Photos[ibase+pictureid].downloads[versionid].description)>0) fprintf(stdout,"%s",Photos[ibase+pictureid].downloads[versionid].description);
                                 }
                                 c=d;
                                 c++;
                            } 
                            else {
                                 *d='}';
                                 d++;
                                 char f=*d;
                                 *d='\0';
                                 fprintf(stdout,"%s",e);
                                 *d=f;
                                 c=d;
                            }
                         }
                         else {
                              fprintf(stdout,"%s",e);
                              *c='\0';
                         }
                    } else {
                         fprintf(stdout,"%s",e);
                         *c='\0';
                    }
               } /* endwhile */
          }
     }
     fclose(f);
} /* endif */
};


// FixSpaces - convert spaces in file names to %20's so that older browsers will render them.
void FixSpaces(char *fn) {
char buf[1024], *lfn, *lbuf;
lfn=fn;
lbuf=buf;
// Catch 0 length string.
while (*lfn!='\0') {
     *lbuf=*lfn;
     if (*lfn==' ') {
          *lbuf='%';
          lbuf++;
          *lbuf='2';
          lbuf++;
          *lbuf='0';
     } /* endif */
     lbuf++;
     lfn++;
} /* endwhile */
*lbuf=*lfn;
strcpy(fn,buf);
};

#define err_inimissing 0
#define err_inimissingt "INI file not found."
#define err_badparm 1
#define err_badparmt "Bad parameter."
#define err_badmax 2
#define err_badmaxt "Bad maximum number of photos in control file."
#define err_memory 3
#define err_memoryt "Out of memory allocating for photos."
#define err_baderrfn 4
#define err_baderrfnt "Bad error file name in control file."
#define err_badmasterfn 5
#define err_badmasterfnt "Bad master file name in control file."
#define err_badlogfn 6
#define err_badlogfnt "Bad log file name in control file."
#define err_toomany 7
#define err_toomanyt "Too many photos, change max number on line one of control file."
#define err_badphoto 8
#define err_badphotot "Bad photo line in control file."
#define err_badlog 9
#define err_badlogt "Unable to access log."
#define err_badcontrol 10
#define err_badcontrolt "name of control file does not match first line of control file."

#define err_unknownerrort "Internal error: Unknown error code."

void ErrorPage(int msgnum, int line) {
char msg[512];
msg[0]='\0';
sprintf(msg,"(%i) ",line);
switch(msgnum) {
     case err_inimissing : strncata(msg,err_inimissingt,512); break;
     case err_badparm : strncata(msg,err_badparmt,512); break;
     case err_badmax : strncata(msg,err_badmaxt,512); break;
     case err_memory : strncata(msg,err_memoryt,512); break;
     case err_baderrfn : strncata(msg,err_baderrfnt,512); break;
     case err_badmasterfn : strncata(msg,err_badmasterfnt,512); break;
     case err_badlogfn : strncata(msg,err_badlogfnt,512); break;
     case err_toomany : strncata(msg,err_toomanyt,512); break;
     case err_badphoto : strncata(msg,err_badphotot,512); break;
     case err_badlog : strncata(msg,err_badlogt,512); break;
     case err_badcontrol : strncata(msg,err_badcontrolt,512); break;
     default: strncata(msg,err_unknownerrort,512); break;
}
ReturnPage(errfn,msg);
};


int ReadSettings() {
char buf[1024];
int line=0;
// Sanity check
if (Photos!=NULL) delete Photos;
Photos=NULL;
// Open file
FILE *f=fopen(settingsfn,"rt");
if (f==NULL) {
     ErrorPage(err_inimissing,line);
     return(-1);
};

// Load name of control file, make sure it matches.
fgets(buf,1024,f);
line++;
char *t=strchr(buf,'\n');
if (t!=NULL) t[0]='\0';
getstring(buf,masterfn);
if ((strlena(masterfn)==0) || (stricmp(masterfn,settingsfn)!=0)) {
     ErrorPage(err_badcontrol,line);
     return(-1);
};

// Read line with maximum number of photos to support
fgets(buf,1024,f);
line++;
int32 maxphotocount;
if (getint(buf,&maxphotocount)!=0) {
     ErrorPage(err_badmax,line);
     return(-1);
};
if (maxphotocount<=0) {
     ErrorPage(err_badmax,line);
     return(-1);
};

// Allocate memory for photos
Photos=new TPhoto[maxphotocount];
if (Photos==NULL) {
     ErrorPage(err_memory,0);
     return(-1);
};

// Load name of output format .HTML page
fgets(buf,1024,f);
line++;
t=strchr(buf,'\n');
if (t!=NULL) t[0]='\0';
getstring(buf,masterfn);
if (strlena(masterfn)==0) {
     ErrorPage(err_badmasterfn,line);
     return(-1);
};

// Load name of error format .HTML page
fgets(buf,1024,f);
line++;
t=strchr(buf,'\n');
if (t!=NULL) t[0]='\0';
getstring(buf,errfn);
if (strlena(errfn)==0) {
     ErrorPage(err_baderrfn,line);
     return(-1);
};

// Load name of log file
fgets(buf,1024,f);
line++;
t=strchr(buf,'\n');
if (t!=NULL) t[0]='\0';
getstring(buf,logfn);
if (strlena(logfn)==0) {
     ErrorPage(err_badlogfn,line);
     return(-1);
};

// Load photos
// ThumbFilename,PreviewFilename,# of other sizes for download,downloadsize0,downloadfile0,downloadsize1,downloadfile1...,downloadsizen,downloadfilen,"Text Description"
iphotocount=0;
boolean done=false;
while ((iphotocount<maxphotocount) && (!done)) {
     // Load line from command file
     fgets(buf,1024,f);
     line++;
     t=strchr(buf,'\n');
     if (t!=NULL) t[0]='\0';
     spudspaces(buf);
     if (strlena(buf)>0) {
          // Parse the fields out
          getstring(buf,Photos[iphotocount].thumbname);
          getstring(buf,Photos[iphotocount].previewname);
          if ((strlena(Photos[iphotocount].thumbname)==0) || (strlena(Photos[iphotocount].previewname)==0)) {
               ErrorPage(err_badphoto,line);
               return(-1);
          };
          if (getint(buf,&Photos[iphotocount].downloadcount)!=0) {
               ErrorPage(err_badphoto,line);
               return(-1);
          };
          if ((Photos[iphotocount].downloadcount<0) || (Photos[iphotocount].downloadcount>MAX_DOWNLOADS))  {
               ErrorPage(err_badphoto,line);
               return(-1);
          };
          FixSpaces(Photos[iphotocount].thumbname);
          FixSpaces(Photos[iphotocount].previewname);
          for (int32 x=0;x<Photos[iphotocount].downloadcount;x++) {
               getstring(buf,Photos[iphotocount].downloads[x].description);
               getstring(buf,Photos[iphotocount].downloads[x].downloadname);
               if ((strlena(Photos[iphotocount].downloads[x].downloadname)==0) || (strlena(Photos[iphotocount].downloads[x].description)==0)) {
                    ErrorPage(err_badphoto,line);
                    return(-1);
               };
               FixSpaces(Photos[iphotocount].downloads[x].downloadname);
          } /* endfor */
          getstring(buf,Photos[iphotocount].description);
          iphotocount++;
     }
     else done=true;
} /* endwhile */
// Check if we overflowed the allocated photo array
if (iphotocount==maxphotocount) {
     ErrorPage(err_badlogfn,0);
     return(-1);
};
fclose(f);
return(0);
};



void main(void) {

Inputs inputs(64,25,1024);
timevalue t;
int32 b;
char *ts, buf[1024], control[PARM_LEN+1], base[PARM_LEN+1], length[PARM_LEN+1], selection[PARM_LEN+1];

// Initialize globals
settingsfn[0]='\0';
logfn[0]='\0';
masterfn[0]='\0';
errfn[0]='\0';
iphotocount=0;
ibase=0;
ilength=0;
inputlength=0;
iselected=0;


// Retrieve IP of caller plus build command line
ip[0]='\0';
if (getenv("REMOTE_ADDR")!=NULL) {
     strncpya(ip,getenv("REMOTE_ADDR"),IP_LEN);
} /* endif */
scriptname[0]='\0';
if (getenv("SCRIPT_NAME")!=NULL) {
     strncata(scriptname,getenv("SCRIPT_NAME"),SCR_LEN);
     strncata(scriptname,"?",SCR_LEN);
} /* endif */
if (inputs["control"]!=NULL) {
     inputs[""];
     strncata(scriptname,"control=",SCR_LEN);
     strncata(scriptname,inputs["control"],SCR_LEN);
} /* endif */
if (inputs["base"]!=NULL) {
     inputs[""];
     strncata(scriptname,"&base=",SCR_LEN);
     strncata(scriptname,inputs["base"],SCR_LEN);
} /* endif */
if (inputs["length"]!=NULL) {
     inputs[""];
     strncata(scriptname,"&length=",SCR_LEN);
     strncata(scriptname,inputs["length"],SCR_LEN);
} /* endif */
if (inputs["selected"]!=NULL) {
     inputs[""];
     strncata(scriptname,"&selected=",SCR_LEN);
     strncata(scriptname,inputs["selected"],SCR_LEN);
} /* endif */

// Check if control parameter is NULL, if so return error page.
inputs[""];                                 
if (inputs["control"]==NULL) {
     ErrorPage(err_badparm,__LINE__);
     return;
} /* endif */

// control parameter is ok, get the value and see if it is one we can use
inputs[""];                                 
strncpya(settingsfn,inputs["control"],PARM_LEN);

if (ReadSettings()!=0) {
     return;
} /* endif */

// Log the command line
if (strlena(logfn)!=0) {
     FILE *l;
     t=TimeGet();
     do {
          l=fopen(logfn,"at");
          if (l==NULL) SysSleep(50);
     } while ((l==NULL) && (TimeElapsed(t)<5000)); /* enddo */
     if (l==NULL) {
          ErrorPage(err_badlog,__LINE__);
          return;
     } /* endif */
     fprintf(l,"%s %s %s\n",ip,TimeStringA(buf,1024),scriptname);
     fclose(l);
};

// Get our input parameters and assign reasonable defaults if not found
inputs[""];
ibase=0;
if (inputs["base"]!=NULL) {
     inputs[""];
     strncpya(buf,inputs["base"],PARM_LEN);
     if (sscanf(buf,"%i",&ibase)!=1) ibase=0;
} 
inputs[""];
ilength=0;
inputlength=0;
if (inputs["length"]!=NULL) {
     inputs[""];
     strncpya(buf,inputs["length"],PARM_LEN);
     if (sscanf(buf,"%i",&ilength)!=1) ilength=10;
     inputlength=ilength;
} 
inputs[""];
iselected=0;
if (inputs["selected"]!=NULL) {
     inputs[""];
     strncpya(buf,inputs["selected"],PARM_LEN);
     if (sscanf(buf,"%i",&iselected)!=1) iselected=0;
} 

// Sanity check parameters
if ((ibase<0) || (ibase>=iphotocount) || (iselected<0)) {
     ErrorPage(err_badparm,__LINE__);
     return;
}

// Make sure length doesn't go beyond end of photo database
if (ibase+ilength>iphotocount) ilength=iphotocount-ibase;

// Make sure selected is in range
if (ibase+iselected>=iphotocount) iselected=iphotocount-ibase-1;

// Write out the result page
ReturnPage(masterfn,"");
};

